@@ -13,7 +13,7 @@ type_refs = {
     "LF_CLASS": ["fieldlist", "derived", "vshape"],
     "LF_ENUM": ["utype", "fieldlist"],
     "LF_FIELDLIST": [],
-    "LF_MFUNC": ["return_type", "class_type", "this_type", "arglist"],
+    "LF_MFUNCTION": ["return_type", "class_type", "this_type", "arglist"],
     "LF_MODIFIER": ["modified_type"],
     "LF_POINTER": ["utype"],
     "LF_PROCEDURE": ["return_type", "arglist"],
@@ -23,6 +23,9 @@ type_refs = {
     "LF_UNION_ST": ["fieldlist"],
     "LF_VTSHAPE": [],
 
+    # TODO: Unparsed
+    "LF_METHODLIST": [],
+
     # FIELDLIST substructures
     "LF_BCLASS": ["index"],
     "LF_ENUMERATE": [],
@@ -691,7 +694,7 @@ subStruct = Struct("substructs",
 )
 
 lfFieldList = Struct("lfFieldList",
-    GreedyRange(subStruct)
+    OptionalGreedyRange(subStruct)
 )
 
 lfEnum = Struct("lfEnum",
@@ -859,8 +862,12 @@ lfVTShape = Struct("lfVTShape",
     BitStruct("vt_descriptors",
         Array(lambda ctx: ctx._.count,
             BitField("vt_descriptors", 4)
-        )
-    )
+        ),
+        # Needed to align to a byte boundary
+        Padding(lambda ctx: (ctx._.count % 2) * 4),
+    ),
+    Peek(ULInt8("_pad")),
+    PadAlign,
 )
 
 Type = Debugger(Struct("type",
@@ -874,7 +881,7 @@ Type = Debugger(Struct("type",
             "LF_CLASS": lfClass,
             "LF_ENUM": lfEnum,
             "LF_FIELDLIST": lfFieldList,
-            "LF_MFUNC": lfMFunc,
+            "LF_MFUNCTION": lfMFunc,
             "LF_MODIFIER": lfModifier,
             "LF_POINTER": lfPointer,
             "LF_PROCEDURE": lfProcedure,
@@ -888,7 +895,7 @@ Type = Debugger(Struct("type",
     ),
 ))
 
-Type = Struct("types",
+Types = Struct("types",
     ULInt16("length"),
     Tunnel(
         String("type_data", lambda ctx: ctx.length),
@@ -925,11 +932,12 @@ Header = Struct("TPIHeader",
 ### Stream as a whole
 TPIStream = Struct("TPIStream",
     Header,
-    Array(lambda ctx: ctx.TPIHeader.ti_max - ctx.TPIHeader.ti_min, Type),
+    Array(lambda ctx: ctx.TPIHeader.ti_max - ctx.TPIHeader.ti_min, Types),
 )
 
 ### END PURE CONSTRUCT DATA ###
 
+# FIXME: this should not be necessary if we use the Embed construct
 def merge_subcon(parent, subattr):
     """Merge a subcon's fields into its parent.
 
@@ -937,8 +945,10 @@ def merge_subcon(parent, subattr):
     subattr: the name of the subconstruct
     """
 
-    subcon = getattr(parent, subattr)
-    for a in (k for k in dir(subcon) if not k.startswith("_")):
+    subcon = getattr(parent, subattr, None)
+    if not subcon: return
+
+    for a in subcon:
         setattr(parent, a, getattr(subcon, a))
 
     delattr(parent, subattr)
@@ -1002,7 +1012,9 @@ def resolve_typerefs(leaf, types, min):
             if ref < min:
                 setattr(leaf, attr, base_type._decode(ref,{}))
             elif ref >= min:
-                setattr(leaf, attr, types[ref])
+                try:
+                    setattr(leaf, attr, types[ref])
+                except KeyError: pass
     return leaf
 
 def merge_fwdrefs(leaf, types, map):
@@ -1017,7 +1029,7 @@ def merge_fwdrefs(leaf, types, map):
             setattr(leaf, attr, newrefs)
         elif not isinstance(ref,str):
             try: newref = types[map[ref.tpi_idx]]
-            except KeyError: newref = ref
+            except (KeyError, AttributeError): newref = ref
             setattr(leaf, attr, newref)
     return leaf
 
